package filter
{
	import flash.display.BitmapData;
	import flash.display.Shader;
	import flash.display.ShaderJob;
	import flash.events.ShaderEvent;
	import flash.geom.ColorTransform;
	
	public class ComplexPatternFilter 
	{
		
		
		/** source image */
		public var	input:BitmapData;	
			
		/**	destination image */
		public var	output:BitmapData;	
		
		/**	shader */
		protected var shader:Shader,
			_motionBlur:Boolean = false;	
		
		/**	embedded .pbj	*/
		[Embed(source='../assets/ComplexInverse.pbj', mimeType = 'application/octet-stream')]
		protected var EmbeddedShader:Class;
		
		
		
		/**	Constructor */
		public function ComplexPatternFilter(
			_input:BitmapData,	
			_output:BitmapData	)
		{
			input=_input;
			output=_output;
			
			shader = new Shader(new EmbeddedShader());
		}
		
		/**	Applies the shader to image */
		public function render():void {
			if (input==null) return;
			shader.data.src.input = input;
			shader.data.imagesize.value = [input.width, input.height];
			
			if (this._motionBlur) {
				
				var bmd:BitmapData = new BitmapData(output.width, output.height, true,0),
					shaderJob:ShaderJob = new ShaderJob(shader, bmd, output.width, output.height);
				//	calls the function finishRender, when shading is completed
				shaderJob.addEventListener(ShaderEvent.COMPLETE, finishRenderTracks);
				shaderJob.start();
				
			} else {
				
				shaderJob = new ShaderJob(shader, output, output.width, output.height);
				output.lock();
				//	calls the function finishRender, when shading is completed
				shaderJob.addEventListener(ShaderEvent.COMPLETE, finishRender);
				shaderJob.start();
				
			}
			
		}
		
		private function finishRender(e:ShaderEvent):void {
			output.unlock();
		}
		private function finishRenderTracks(e:ShaderEvent):void {
			output.lock();
			//	fade out the earlier bitmap
			output.colorTransform(output.rect, new ColorTransform(1,1,1,7/8));
			//	draw the new rendering upon it
			output.draw(e.bitmapData);
			output.unlock();
		}
		
		
		/** Amount of pattern fill */
		public function set fill (value:Number):void {
			shader.data.fill.value = [value];
			render();
		}
		
		/**	Set the background color for areas between the circular cells */
		public function set bgcolor(value:uint):void {
			shader.data.bgcolor.value= [
				((value>>16)&0xFF)/255,
				((value>>8)&0xFF)/255,
				((value)&0xFF)/255,
				((value>>24)&0xFF)/255
				]; 
			render();
		}
		
		
		/**	does the previous image appear on bitmap */
		public function set motionBlur(value:Boolean):void {
			this._motionBlur = value;
			render();
		}
		
		
		/**	Base point for pattern origin, parameter point: any kind of object with properties x and y */
		public function set center(point:Object):void { 
			shader.data.center.value = [ point.x, point.y ]; 
			render();
		}
		public function get center():Object {
			return {x:shader.data.center.value[0], y:shader.data.center.value[1]};
		}
		
		
		/**	point a, parameter point: any kind of object with properties x and y */
		public function set pointA(point:Object):void { 
			shader.data.a.value = this.screenToFractalCoordinates(point); 
			render();
		}
		public function get pointA():Object {
			return this.fractalToScreenCoordinates(shader.data.a.value);
		}
		
		/**	point b, parameter point: any kind of object with properties x and y */
		public function set pointB(point:Object):void { 
			shader.data.b.value = this.screenToFractalCoordinates(point); 
			render();
		}
		public function get pointB():Object {
			return this.fractalToScreenCoordinates(shader.data.b.value);
		}
		
		/**	point c, parameter point: any kind of object with properties x and y */
		public function set pointC(point:Object):void { 
			shader.data.c.value = this.screenToFractalCoordinates(point); 
			render();
		}
		public function get pointC():Object {
			return this.fractalToScreenCoordinates(shader.data.c.value);
		}
		
		
		private function screenToFractalCoordinates (point:Object):Array {
			var c:Object = this.center,
				sca:Number = this.scale;
			return [ (point.x-c.x)/sca, (point.y-c.y)/sca ]; 
		}
		
		private function fractalToScreenCoordinates (point:Array):Object {
			var c:Object = this.center,
				sca:Number = this.scale;
			return { x:point[0]*sca+c.x, y:point[0]*sca+c.y }; 
		}
		
		
		/**	parameter scale sets how concentrated the circles are around the center */
		public function set scale(value:Number):void {
			shader.data.scale.value= [value]; 
			render();
		}
		public function get scale():Number {
			return shader.data.scale.value[0];
		}
		
		
		/**	Rotates the shapes around the focal points */
		public function set rotation(value:Number):void {
			var c:Number = Math.cos(value),
				s:Number = Math.sin(value);
			shader.data.d.value= [c,s]; 
			render();
		}
		
		/**	set pattern scaling in x-direction */
		public function set distortX(value:Number):void {
			shader.data.distort.value[0]= value; 
			render();
		}
		
		/**	*/
		public function set focus(value:Number):void { 
			shader.data.focus.value = [ value ]; 
			render();
		}
		
		
		/**	set pattern scaling in y-direction */
		public function set distortY(value:Number):void {
			shader.data.distort.value[1]= value; 
			render();
		}
		
	}
}